@@ -9,9 +9,9 @@
 try:
     from types import *
     import string, re, os
-    import Globals
     from AccessControl import ClassSecurityInfo
     from Acquisition import aq_base, aq_inner, aq_parent
+    from Globals import InitializeClass
     from OFS.DTMLDocument import DTMLDocument
     from Products.CMFCore.PortalContent import PortalContent
     from Products.CMFCore.utils import _getViewFor
@@ -30,7 +30,8 @@ try:
         security.declarePublic('allowedContentTypes')
         def allowedContentTypes( self ):
             return []
-
+    InitializeClass(WikiFolder)
+        
     class ZwikiDublinCoreImpl(DefaultDublinCoreImpl):
         """
         Zwiki's implementation of Dublin Core.
@@ -62,6 +63,20 @@ try:
         def CreationDate(self):
             return self.creationTime().ISO()
 
+        security.declarePublic( 'getPageTitle' )
+        def getPageTitle(self, here=None, template=None, portal_title=None):
+            """
+            Return the proper name of this page for use by Plone.
+
+            A quick fix for plone's script, which doesn't like using
+            our page's title when it's the same as the id.
+            XXX file a bug
+            """
+            return self.pageName()
+        
+
+    InitializeClass(ZwikiDublinCoreImpl)
+
     class CMFAwareness(PortalContent, ZwikiDublinCoreImpl):
         """
         Mix-in class for CMF support
@@ -75,7 +90,6 @@ try:
         set = security.setPermissionDefault
         set(Permissions.Edit, ('Owner', 'Manager', 'Authenticated'))
         set(Permissions.FTPRead, ('Owner', 'Manager'))
-        set(Permissions.ChangeRegs, ('Owner', 'Manager'))
         set(Permissions.Add, ('Owner', 'Manager', 'Authenticated'))
         #set(Permissions.Move, ('Owner', 'Manager'))
         set(Permissions.Comment, ('Owner', 'Manager', 'Authenticated'))
@@ -107,30 +121,21 @@ try:
         def SearchableText(self):
             return self.text()
 
-        security.declareProtected(Permissions.View, '__call__')
-        def __call__(self, client=None, REQUEST={}, RESPONSE=None, **kw):
-            '''
-            Invokes the default view.
-            '''
-            view = _getViewFor( self )
-            if getattr(aq_base(view), 'isDocTemp', 0):
-                return apply(view, (self, REQUEST))
-            else:
-                if REQUEST:
-                    kw[ 'REQUEST' ] = REQUEST
-                if RESPONSE:
-                    kw[ 'RESPONSE' ] = RESPONSE
-                return apply( view, (self,), kw )
+        security.declareProtected(Permissions.View, 'Subject')
+        def Subject(self):
+            return self.spacedPageName()
 
-        # needed ?
-        #index_html = None  # This special value informs ZPublisher to use __call__
+        security.declareProtected(Permissions.View, 'Description')
+        def Description(self):
+            return self.summary()
 
         def wiki_context(self, REQUEST=None, with_siblings=0):
             return self.context(REQUEST, with_siblings)
 
-    Globals.InitializeClass(CMFAwareness) # install permissions
-
 except ImportError:
     class CMFAwareness:
         def supportsCMF(self): return 0
         def inCMF(self): return 0
+
+InitializeClass(CMFAwareness)
+
